@@ -36,7 +36,7 @@
 #include <stdarg.h>
 #include <errno.h>
 
-#include <gobject/gobject.h>
+#include <glib-object.h>
 
 
 G_BEGIN_DECLS
@@ -81,6 +81,10 @@ struct _GnomeProgram
 struct _GnomeProgramClass
 {
     GObjectClass object_class;
+
+    /* we may want to add stuff in the future */
+    gpointer padding1;
+    gpointer padding2;
 };
 
 GType
@@ -105,8 +109,7 @@ gnome_program_locate_file               (GnomeProgram    *program,
 					 gboolean         only_if_exists,
 					 GSList         **ret_locations);
 
-#define GNOME_PARAM_MODULE_INFO         "module-info"
-#define GNOME_PARAM_MODULES             "modules"
+#define GNOME_PARAM_NONE                NULL
 #define GNOME_PARAM_POPT_TABLE          "popt-table"
 #define GNOME_PARAM_POPT_FLAGS          "popt-flags"
 #define GNOME_PARAM_POPT_CONTEXT        "popt-context"
@@ -147,7 +150,9 @@ typedef void (*GnomeModuleHook) (GnomeProgram *program,
 				 GnomeModuleInfo *mod_info);
 
 struct _GnomeModuleInfo {
-    const char *name, *version, *description;
+    const char *name;
+    const char *version;
+    const char *description;
     GnomeModuleRequirement *requirements; /* last element has NULL version */
 
     GnomeModuleHook instance_init;
@@ -155,15 +160,12 @@ struct _GnomeModuleInfo {
 
     struct poptOption *options;
 
-    GnomeModuleInitHook init_pass; /* This gets run before the preinit
-				      function to allow the module to
-				      register other modules as needed. The
-				      module cannot assume its required
-				      modules are initialized (they aren't). */
+    GnomeModuleInitHook init_pass;
 
     GnomeModuleClassInitHook class_init;
 
-    gpointer expansion1, expansion2;
+    const char *opt_prefix;
+    gpointer    expansion1;
 };
 
 /* This function should be called before gnomelib_preinit() - it's an
@@ -223,16 +225,12 @@ gnome_program_initv (GType type,
 		     const GnomeModuleInfo *module_info,
 		     int argc, char **argv,
 		     const char *first_property_name, va_list args);
-
-
-/* Some systems, like Red Hat 4.0, define these but don't declare
-   them.  Hopefully it is safe to always declare them here.  */
-#ifndef program_invocation_short_name
-char *program_invocation_short_name;
-#endif
-#ifndef program_invocation_name
-char *program_invocation_name;
-#endif
+GnomeProgram*
+gnome_program_init_paramv (GType type,
+                           const char *app_id, const char *app_version,
+                           const GnomeModuleInfo *module_info,
+                           int argc, char **argv,
+                           guint nparams, GParameter *params);
 
 G_END_DECLS
 
